#!/usr/bin/env node

import inquirer from "inquirer";
import path from "path";
import * as fs from "fs";
import { installUmbraco } from "./umbraco/installUmbraco.js";
import { executeInSeries } from "./utils/utils.js";
import { setupFrontend } from "./frontend/setupFrontend.js";
import { initializeGitRepo } from "./generic/initializeGitRepo.js";
import { installDotNetSDK } from "./umbraco/installDotNetSDK.js";
import { createSolutionFile } from "./umbraco/createSolutionFile.js";

const setupProject = async () => {
  try {
    const answers: any = await inquirer.prompt([
      {
        type: "input",
        name: "projectName",
        message: "Enter the project name:",
        validate: (input: string) => (input ? true : "Project name cannot be empty"),
      },
      {
        type: "confirm",
        name: "useUmbraco",
        message: "Do you want to set up the project with Umbraco?",
        default: true,
      },
      {
        type: "list",
        name: "versionChoice",
        message: "Choose the Umbraco version:",
        choices: ["Latest", "Other"],
        when: (answers: any) => answers.useUmbraco,
      },
      {
        type: "list",
        name: "umbracoVersion",
        message: "Choose the Umbraco version:",
        choices: ["13.5.1", "12.3.10", "11.5.0", "10.8.6", "Choose"],
        when: (answers: any) => answers.versionChoice === "Other",
      },
      {
        type: "input",
        name: "ChooseUmbracoVersion",
        message: "Write the Umbraco version:",
        when: (answers: any) => answers.umbracoVersion === "Choose",
        validate: (input: string) => (input ? true : "Umbraco version cannot be empty"),
      },
      {
        type: "list",
        name: "frontendTech",
        message: "Choose the frontend technology:",
        choices: ["Gulp", "Vue", "React"],
      },
      {
        type: "input",
        name: "gitRepo",
        message: "Enter the Git repository URL: (Optional)",
      },
    ]);

    const umbracoVersion =
      answers.versionChoice === "Other"
        ? answers.umbracoVersion === "Choose"
          ? answers.ChooseUmbracoVersion
          : answers.umbracoVersion
        : answers.versionChoice;

    const projectDir = path.join(process.cwd(), answers.projectName);
    const umbracoDir = path.join(projectDir, answers.projectName);

    console.log(`Creating project directory at ${projectDir}...\n`);

    if (!fs.existsSync(projectDir)) {
      fs.mkdirSync(projectDir);
    }

    console.log(`Setting up project in ${projectDir}...\n`);

    const tasks = [
      () => setupFrontend(projectDir, answers),
      () => initializeGitRepo(projectDir, answers),
    ];

    if (answers.useUmbraco) {
      tasks.unshift(
        () => installDotNetSDK(),
        () => installUmbraco(umbracoDir, umbracoVersion),
        () => createSolutionFile(projectDir, umbracoDir, answers)
      );
    }

    await executeInSeries(tasks);
  } catch (err) {
    console.error("Error setting up project:", err);
  }
};

setupProject();
